#include <QDebug>

#include "serialcontroller.h"
#include "common.h"

serialcontroller::serialcontroller(QObject *parent) : QObject(parent)
{
    //
    m_port.setPortName("/dev/ttymxc1");
    m_port.setBaudRate(QSerialPort::Baud115200, QSerialPort::AllDirections);
    m_port.setDataBits(QSerialPort::Data8);
    m_port.setParity(QSerialPort::NoParity);
    m_port.setStopBits(QSerialPort::OneStop);
    m_port.setFlowControl(QSerialPort::NoFlowControl);

    //open serial connection
    m_port.open(QIODevice::ReadWrite);
    qDebug() << "\nSerial Connection: "<<m_port.isOpen();

    //checks serial connection is open, and if so echoes the serial parameters
   if (m_port.isOpen()) {
       qDebug() << "Serial port " << m_port.portName() << " is open";
       qDebug() << "Baud rate" << m_port.baudRate();
       qDebug() << "Data bits" << m_port.dataBits();
       qDebug() << "Stop bits" << m_port.stopBits();
       qDebug() << "Parity" << m_port.parity();

       //sets up a signal and slot for when there is data on the serial line, it runs the onSerialReadyRead function.
       connect(&m_port, &QSerialPort::readyRead, this, &serialcontroller::onSerialReadyRead );

       //This sends data to the FDA board to let it know that the serial connection is set up and ready to operate.
       //which in turn prompts the FDA board to send the initial information to initalize the screen
       m_port.write("\0xFF");
       m_port.write("\0x00");
       m_port.write("YK");
   } else {
       qDebug() << "Serial port error: Could not open" << m_port.portName() << " : " << m_port.errorString();
   }
}

serialcontroller::~serialcontroller()
{
    qDebug() << "Closing serial port";
    m_port.close();
}

void serialcontroller::send(QString msg)
{
    qDebug() << "[qml] Sent to micro --> " <<msg << " <--";
    QByteArray sendmsg = msg.toLatin1();
    m_port.write(sendmsg);
}

void serialcontroller::onSerialReadyRead()
{
    QByteArray rcvd = m_port.readAll();
    qDebug()<<"Received: "<<rcvd;
    //Process incoming string 1 3byte hex string at a time
    for(int i=0;i<rcvd.size();i=i+3){
        QByteArray current = rcvd.mid(i,3);
        qDebug()<<"current: "<<current;
        if(current[0]=='\x82'){
            int rcvVal = current[2];
            qDebug()<<"rcvVal: "<<rcvVal;

            //Cases for the response from the LEI circuitboard
            //11: operate
            if(rcvVal=='\x0B'){
                emit readyStandby(true); //Set mode to Ready
            }
            //19: restore Standby
            if(rcvVal=='\x13'){
                emit readyStandby(false);
            }
            //20 Lasing
            if(rcvVal==20){
                emit lasing(true);
            }
            //21 not Lasing
            if(rcvVal==21){
                emit lasing(false);
            }
            //40 Error Messages & 49 Error Reset
            if((rcvVal>=40)&&(rcvVal<=49)){
                emit error(rcvVal);
            }
            //60-71: Pulse, 71 Pulse Off, 60 Half sec
            if((rcvVal>=60)&&(rcvVal<=71)){
                double pulse = 0;
                pulse = (rcvVal-60);
                if(pulse==0){pulse=0.5;}
                //qDebug()<<pulse;
                if(pulse==11){
                    emit pulseUpdate("CONT");
                    emit repUpdate("OFF");
                } else{
                    emit pulseUpdate(pulse/10);
                }
            }
            //81-91: repeat, 91 repeat off
            if((rcvVal>80)&&(rcvVal<=91)){
                double rep = 0;
                rep = (rcvVal-80);
                if(rep==11){
                    emit repUpdate("OFF");
                } else{
                    emit repUpdate(rep/10);
                }
            }
            //93CW no pulse, 94 CW single pulse, 95 CW pulses
            if((rcvVal>=93)&&(rcvVal<=95)){ emit modeChange(true, rcvVal); }
            //96SP no pulse, 97 SP single pulse, 98 SP pulsing
            if((rcvVal>=96)&&(rcvVal<=98)){ emit modeChange(false, rcvVal); }
            //100-180: Power in Watts. 100 = halfwatt
            if((rcvVal>=100)&&(rcvVal<=180)){
                double pow = 0;
                pow = rcvVal-100;
                if(pow==0){pow=0.5;}
                emit powerUpdate(pow);
            }

            m_port.write("\x3E\x0D");
            qDebug() <<"Handshake Sent";
            //Follow-up code to put display into Ready mode
            if(current[2]=='\x0B'){
                m_port.write("YM");
                qDebug() << "[serialcontroller.cpp] Sent to micro --> YM <--";
            }
            //Follow-up code to put display into Standby Mode.
            if(current[2]=='\x13'){
                m_port.write("YL");
                qDebug() << "[serialcontroller.cpp] Sent to micro --> YL <--";
            }
        }
    }
}
